#ifndef __XDEVICE_ATTRIBUTIONS_H__
#define __XDEVICE_ATTRIBUTIONS_H__

#include <stdint.h>


#define INVALID_HANDLE 0
typedef uint64_t xhandle_t;
typedef int xerror_t;

///////////////////////////////////////////////////////
/// @enum XControllerButtonMasks
/// @brief Masks of Controller buttons.
// Reference : https://msdn.microsoft.com/en-us/library/windows/apps/microsoft.directx_sdk.reference.xinput_gamepad
enum XControllerButtonMasks {
    kXControllerButton_DpapUp = 0x0001,
    kXControllerButton_DpapDown = 0x0002,
    kXControllerButton_DpadLeft = 0x0004,
    kXControllerButton_DpadRight = 0x0008,
    kXControllerButton_Start = 0x0010,
    kXControllerButton_Back = 0x0020,
    kXControllerButton_LeftThumb = 0x0040,
    kXControllerButton_RightThumb = 0x0080,
    kXControllerButton_LeftShoulder = 0x0100,
    kXControllerButton_RightShoulder = 0x0200,
    kXControllerButton_Guide = 0x0400,
    kXControllerButton_A = 0x1000,
    kXControllerButton_B = 0x2000,
    kXControllerButton_X = 0x4000,
    kXControllerButton_Y = 0x8000,
    // Emulation
    kXControllerButton_LeftThumbMove = 0x10000,
    kXControllerButton_RightThumbMove = 0x20000,
    kXControllerButton_LeftTrigger = 0x40000,
    kXControllerButton_RightTrigger = 0x80000,
    kXControllerButton_LeftThumbUp = 0x100000,
    kXControllerButton_LeftThumbDown = 0x200000,
    kXControllerButton_LeftThumbLeft = 0x400000,
    kXControllerButton_LeftThumbRight = 0x800000,
    kXControllerButton_RightThumbUp = 0x1000000,
    kXControllerButton_RightThumbDown = 0x2000000,
    kXControllerButton_RightThumbLeft = 0x4000000,
    kXControllerButton_RightThumbRight = 0x8000000,
    //
    kXControllerButton_None = 0x0,
    kXControllerButton_ANY = ~kXControllerButton_None,
};

enum XContextStates {
    kXContextStInited, ///< context state, context enviroment allocted, but not working.
    kXContextStStarted, ///< context state, devices in context working.
    kXContextStWillRelease, ///< context state, moment before release.
};

////////////////////////////////////////////////////////
/// @enum XContextAttributes
/// @brief Attributes of Context
/// @deprecated 
enum XContextAttributes {
    //kXCtxAttr_DeviceVersion,
    kXCtxAttr_Int_SdkVersion,
    kXCtxAttr_Str_SdkVersion,
    kXCtxAttr_Int_SDKALGVersion, // int
    kXCtxAttr_Int_State, ///< State value indicated context working state, 
    kXCtxAttr_Str_SdkBuildDetail,
};

////////////////////////////////////////////////////////
/// @enum XVpuAttributes
/// @brief Attributes of VPU Device.
/// @deprecated 
enum XVpuAttributes {
    kXVpuAttr_Str_SoftwareRevision,
    kXVpuAttr_Str_HardwareRevision, 
    kXVpuAttr_Str_SerialNumber,
    kXVpuAttr_Str_DeviceName,
    kXVpuAttr_Str_ModelName,
    kXVpuAttr_Str_FPGAVersion,
    kXVpuAttr_Str_ALGVersion,

    kXVpuAttr_Int_ImuFps,
    kXVpuAttr_Int_FpgaFps,
    /// Reversed
    kXVpuAttr_Int_AlgorithmPoseFps,

    kXVpuAttr_Int_ErrorCode,

    /// Connection state of Controller, see \ref XConnectionStates
    kXVpuAttr_Int_ConnectionState,
    
    kXVpuAttr_Int_PowerMode,
    /// Battery Level. Invalid if battery mode is external power
    kXVpuAttr_Int_Battery,
    kXVpuAttr_Int_BatteryVoltage,
    kXVpuAttr_Int_BatteryTemperature,

    /// IMU info of device, see \ref XAttrImuInfo
    kXVpuAttr_Obj_ImuInfo, 
    /// IMU info of device, Output to variable aguments
    kXVpuAttr_V_ImuInfo, 

    /// 6Dof info of device, see \ref XAttr6DofInfo
    kXVpuAttr_Obj_6DofInfo,
    /// 6Dof info of device, Output to variable aguments
    kXVpuAttr_V_6DofInfo,
    
    /// Pressed button bits
    kXVpuAttr_Int_ButtonBits, // int 

    /// Button events, \ref XButtonEvents
    kXVpuAttr_Int_ButtonEvent = kXVpuAttr_Int_ButtonBits,

    /// Number of Paired controllers
    kXVpuAttr_Int_PairedNumber,

    /// Device infos, imu, 6dof, buttons ... see \ref XAttrControllerState
    kXVpuAttr_Obj_ControllerState,

    /// VPU cammera tracking object pose info. see \ref XAttrTrackingInfo
    kXVpuAttr_Obj_TrackingInfo,

    /// VPU cammera tracking object pose info. Output to variable aguments
    kXVpuAttr_V_TrackingInfo,

    //kXVpuAttr_Evt_TrackingPoseUpdate,

    /// VPU paired controllers, see \ref XAttrPairedInfos
    kXVpuAttr_Obj_PairedInfos,

};

////////////////////////////////////////////////////////
/// @enum XControllerAttributes
/// @brief Attribute types of controller.
/// @deprecated 
enum XControllerAttributes {
    kXCAttr_Str_FirmwareRevision,
    kXCAttr_Str_SoftwareRevision,
    kXCAttr_Str_HardwareRevision,
    kXCAttr_Str_SerialNumber,
    kXCAttr_Str_DeviceName,
    kXCAttr_Str_ModelName, 
    kXCAttr_Str_ManufacturerName,

    /// FPS of controller device reporting IMU data
    kXCAttr_Int_ImuFps,

    kXCAttr_Int_ErrorCode,

    /// Connection state of Controller, see \ref XConnectionStates
    kXCAttr_Int_ConnectionState,

    kXCAttr_Int_PowerMode,
    /// Battery Level. Invalid if battery mode is external power
    kXCAttr_Int_Battery,
    kXCAttr_Int_BatteryVoltage,
    kXCAttr_Int_BatteryTemperature,


    /// IMU info of device, see \ref XAttrImuInfo
    kXCAttr_Obj_ImuInfo,
    /// IMU info of device, Output to variable aguments
    kXCAttr_V_ImuInfo,

    /// 6Dof info of device, see \ref XAttr6DofInfo
    kXCAttr_Obj_6DofInfo,
    /// 6Dof info of device, Output to variable aguments
    kXCAttr_V_6DofInfo,

    /// Pressed button bits
    kXCAttr_Int_ButtonBits, 
    /// Button events, \ref XButtonEvents
    kXCAttr_Int_ButtonEvent = kXCAttr_Int_ButtonBits,

    kXCAttr_Int_Trigger,
    /// Device touchpad state info. see \ref XAttrTouchPadInfo
    kXCAttr_Obj_TouchPadState,


    //kXCAttr_Int_TouchPadState, // 

    /// Device infos, imu, 6dof, buttons ... see \ref XAttrControllerState
    kXCAttr_Obj_ControllerState, 

    /// Get Controller device type, see \ref XControllerTypes.
    kXCAttr_Int_Type,
    /// Get Controller device address (MAC address)
    kXCAttr_Obj_Address,
    /// Get Hex String MAC Address of controller.
    kXCAttr_Str_Address,

    /// Get Bind ID of connected controller.
    kXCAttr_Int_BindID, 

};
enum XPowerModes {
    kXPowerModeExternal = 0, ///< External power supply
    kXPowerModeBattery = 1, ///< battery 
};
enum XControllerTypes {
    kControllerType_Saber = 0x0000,
    kControllerType_Kylo = 0x0001,
    kControllerType_DType = 0x0002,
    kControllerType_PickUp = 0x0003,

    kControllerType_3Dof = 0x0200,
    kControllerType_Tag = 0x0100,

    kControllerType_TagLeft = kControllerType_Tag,
    kControllerType_TagRight = 0x0101,
    kControllerType_TagRfid = 0x0102,
    kControllerType_Cube = 0x0103,
    kControllerType_TouchPad = 0x0104,

    kControllerType_ShortGun = 0x0105,
    kControllerType_LongGun = 0x0106,

    kControllerType_CircularLeft = 0x010A,
    kControllerType_CircularRight = 0x010B,

    kControllerType_LedLeft = 0x010C,
    kControllerType_LedRight = 0x010D,
    
    kControllerType_GunElec_95_1 = 0x0115,
    kControllerType_GunAir_95_1 = 0x0116,
    kControllerType_GunElec_92G = 0x0117,
    kControllerType_GunAir_92G = 0x0118,
    kControllerType_GunElec_191 = 0x0119,
    kControllerType_GunAir_191 = 0x011A,

    kControllerType_Unknow = 0xffff,
    kControllerType_BitMask = 0xffff,
};

////////////////////////////////////////////////////////
/// @enum XControllerTrackModes
/// @brief Algorithm tracking types of controller.
enum XControllerTrackModes {
    kControllerTrackModeNone,
    kControllerTrackModeVisable,
    kControllerTrackModeTagMark,
};

//////////////////////////////////////
/// @enum XButtonEvents
/// @brief Button events
enum XButtonEvents {
    kXBtnEvt_Down,
    kXBtnEvt_Up,
    //kXBtnEvt_LongPress,
};

///////////////////////////////////////
/// @struct XButtonEventParam
/// @brief Parameters for button event notification
struct XButtonEventParam {
    /// Button key value.
    uint32_t btn;
    /// Event value. see \ref XButtonEvents
    XButtonEvents evt;
};

////////////////////////////////////////
/// @enum XConnectionStates
/// @brief Connection state of device
enum XConnectionStates {
    kXConnSt_Disconnected = 0,
    kXConnSt_Scanning = 1,
    kXConnSt_Connecting = 2,
    kXConnSt_Connected = 3,
    kXConnSt_Disconnecting =4,
    //kXConnSt_Disconnected = ?
};
enum XPairingStates {
    kXPairStUnpaired = 0,
    kXPairStPairing = 1,
    kXPairStPaired = 2,
    kXPairStUnpairing = 3,
};

enum XTrackResults {
    kXTrackResultNone = 0,
    kXTrackResultPose = 1,
    kXTrackResultPosition = 2,

    kXTrackResultAll = kXTrackResultPosition | kXTrackResultPose,
};
///////////////////////////////////////////////
/// @struct XAttr6DofInfo
/// @brief Structure for gettting 6Dof information
struct XAttr6DofInfo {
    float position[3];  /// < float buffer for gettting position values
    
    float rotation[4]; /// < float buffer for getting quaternions of rotation. (ctrl imu: wxyz/ fusion: xyzw)
    //float quaternion[4];

    uint64_t timestamp;
    float euler[3]; // pitch, yaw, roll
};

//////////////////////////////////////////////
/// @struct XAttrImuInfo
/// @brief Structure for getting IMU information.
struct XAttrImuInfo {
    float accelerometer[3]; ///< (out) float buffer for getting accelerometer values.
    float gyroscope[3]; ///< float buffer for getting gyroscope values.
    float magnetism[3]; /// < float buffer for getting magnetism values.
    uint64_t timestamp;
};

/////////////////////////////////////////////
/// @struct XAttrTrackingInfo
/// @brief VPU cammera tracking object pose info.
struct XAttrTrackingInfo {
    int index;
    int state;
    float position[3];
    float rotation[4]; //< x,y,z,w
    uint64_t timestamp;
    //int n_recognized;
    uint64_t recognized_markers_mask; ///< bit not zero indicated which id recognized. @deprecated
    double confidence;
    double marker_distance;
};


///////////////////////////////////////////////
///// @struct XAttrTrackingInfo
///// @brief VPU cammera tracking object pose info.
//struct XAttrConnectInfo {
//    int index;
//    int state;
//    float position[3];
//    float rotation[4];
//    uint64_t timestamp;
//    //int n_recognized;
//    uint64_t recognized_markers_mask; ///< bit not zero indicated which id recognized.
//};

/////////////////////////////////
/// \brief structure for containing base information of controller.
struct XControllerInfo {
    int type; ///< Controller type
    int bind_id; ///< Bind ID indicate which Controller. @deprecated use index
    bool is_paired; ///< Paired flag.
    bool is_connected; /// Connection state, set \ref XConnectionState
    uint8_t mac[6];
    int index;
};

//////////////////////////////////////
/// @struct XAttrControllerState
/// @brief Structure for getting controller state informations.
struct XAttrControllerState {

    /// quaternion 
    float rotation[4];

    /// x, y, z
    float position[3];
    float accelerometer[3];
    float gyroscope[3];

    uint64_t timestamp;
    /// bit map indicating button pressed state of controller.
    uint32_t button_state;

    //uint8_t valid_flag; /// 
    // TouchPad, reverve
    float axes[6];

    float euler[3];

    int trigger; ///< 0-255
    int grip;
};

enum XAttrTouchPadEvents {
    kAttrTpadEvt_Idle = 0x00,
    kAttrTpadEvt_Press = 0x01,
    kAttrTpadEvt_Release = 0x02,
    kAttrTpadEvt_Move = 0x03,
};
struct XAttrTouchPadState {
    bool pressed;
    float x;
    float y;
};

/////////////////////////////
/// \struct XAttrPairedInfos
/// \bref Structure for getting infomation of paired controllers.
struct XAttrPairedInfos {
    int c; ///< Number of paired controller.
    int bind_id_a[16]; ///< An array containing bind ID of paired controllers.
    uint8_t mac[6][16]; ///< An array containing MAC address of paired controllers.
};

////////////////////////
/// \deprecated
typedef void(*XContextStateChangeDelegateFn_t)(int st, void* ud);

////////////////////////
/// \deprecated
typedef void(*XDeviceConnectStateChangeDelegateFn_t)(int connect_st, void* ud);

////////////////////////
/// \deprecated
typedef void(*XDeviceButtonStateChangeDelegateFn_t)(int btn_mask, int evt, void* ud);

////////////////////////
/// \deprecated
typedef void(*XDeviceTrackingUpdateDelegateFn_t)(void* ud);

/**
 * @brief
 * @deprecated description in client SDK
 */
typedef void(*XContextStateChangeDelegateFn2_t)(void* context, int st, void* ud);

/**
 * @brief
 * @deprecated description in client SDK
 */
typedef void(*XDeviceConnectStateChangeDelegateFn2_t)(void* device, int st, void* ud);

/**
 * @brief
 * @deprecated description in client SDK
 */
typedef void(*XDeviceButtonStateChangeDelegateFn2_t)(void* device, int button_bitsmask, int evt, void* ud);

enum XFirmwareProjectTypes {
    kXFwProjTypeSlideIn,
    kXFwProjTypeTagModule,
    kXFwProjTypeAllInOne,
    kXFwProjTypeAioPro,

    kXFwProjTypeMax,
};
enum XFirmwareTypes {
    kXFwTypeMcu = 0x00,
    kXFwTypeBleHost = 0x01,
    kXFwTypeFpgaIrLed = 0x02,
    kXFwTypeFpgaIrAlg = 0x03,
    kXFwTypeFpgaCalib = 0x04,
};

enum XFirmwareUpgradeEvents{
    kXFwUpgradeEvtBegin,
    kXFwUpgradeEvtTransferProgress,
    kXFwUpgradeEvtEndSuccess,
    kXFwUpgradeEvtEndFailure,
};
enum XFirmwareUpgradeFailReasons {
    kXFwReasonFileNotValid,
    kXFwReasonEnterBootFail,
    kXFwReasonBootAuthFail,
    kXFwReasonStartCommandFail,
    kXFwReasonTransferFail,
    kXFwReasonLoadFirmwareFail,
    kXFwReasonWaitTimeout,
};
typedef void(*XDeviceFirmwareUpgradeEventsDelegateFn_t)(int ev, int int_arg, void* ud);
// typedef void(*XDeviceFirmwareUpgradeEventsDelegateFn_t)(int32_t st, int64_t val, void* ud);
enum XErrorCodes {
    kErrNoError = 0,
    kErrUnknonw = -1,

    kErrBegin = -1000,

    kErrOperationFail,
    kErrDeviceNotFound,
    kErrInvalidArgument,
    kErrVpuNotArrived,
    kErrFileNotExist,
    kErrFileEmpty,
    kErrChecksumFail,
    kErrInWrongState,

    kErrVpuAuthFailed,

    kErrReadMarkerConfigFailed,

    kErrNotFound, // no usage

    kErrApiInvalid,

    kErrVpuCommandFail,

    kErrNoDeviceToConnect,
    kErrWrongConnectState,
    kErrWrongPairingState,

    kErrPairListIsFull,

    kErrWaitTimeout,
    kErrBindIdInvalid,

    kErrOutoffMemory,

    kErrServiceNotRunning,
    kErrServiceInvalid,
    kErrCodeError,

    // kErrOpClient = -2000,

    kErrEnd,

};

enum XEvents {
    kXEvtVpuRequestSuccess,
    kXEvtVpuRequestFailed,

    kXEvtVpuFwUpgradingStart = 100,
    kXEvtVpuFwUpgradingProgress,
    kXEvtVpuFwUpgradingEndSuccess,
    kXEvtVpuFwUpgradingEndFail,

    kXEvtAuthResult = 200,
    kXEvtJavaBroadcast = 201,
    kXEvtJavaOpcReport = 202,
};
//extern const char* GetErrorName(XErrorCodes code);
#endif // __XDEVICE_ATTRIBUTIONS_H__
