/**
 * @file xdevice_client.h
 * @author Rp (renpeng.zhong@ximmerse.com)
 * @brief 
 * @version 1.0     
 *      First version.
 * @date 2022-03-01
 * 
 * @copyright Copyright (c) Ximmerse Ltd., 2022
 * 
 * @~Chinese
 * @defgroup client_1_3_2_33 Client 版本  1.3.2.33
 */

#ifndef __XDEVICE_CLIENT_H__
#define __XDEVICE_CLIENT_H__

#include <stdint.h>

#include "xdevice_attributions.h"

#ifndef XIM_API
#if defined(_WIN32)||defined(_WIN64)
#   if defined(XIM_BUILDING)
#       define XIM_API __declspec(dllexport)
#   else // XIM_BUILDING
#       define XIM_API __declspec(dllimport)
#   endif // XIM_BUILDING
#else
#define XIM_API
#endif
#endif // XIM_API
#ifndef DEPRECATED
#ifdef _MSC_VER  // Visual Studio
#define FORCE_INLINE  // __forceinline causes C4714 when returning std::string
#define NO_INLINE __declspec(noinline)
#define DEPRECATED(msg) __declspec(deprecated(msg))
#elif defined(__GNUC__)  // GCC or Clang
#define FORCE_INLINE __attribute__((always_inline))
#define NO_INLINE __attribute__((noinline))
#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5)
#define DEPRECATED(msg) __attribute__((deprecated(msg)))
#else
#define DEPRECATED(msg) __attribute__((deprecated))
#endif
#else  // Other compilers
#define FORCE_INLINE
#define NO_INLINE
#define DEPRECATED(msg)
#endif
#endif // DEPRECATED

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

typedef enum {
    kXEvtServiceConnected,
    kXEvtServiceDisconnected,

    kXClientEventMax,

    kXEvtNormalEventsBegin = 1000,
    kXEvtInitSuccess = kXEvtNormalEventsBegin,
    kXEvtInitFailed,

    kXEvtConnectionStateChange,
    
    kXEvtDeviceBatteryStateChange,

    kXEvtVpuInfoUpdated,
    
    kXEvtControllerInfoUpdate,

    kXEvtButtonStateChange,
    kXEvtControllerPairStateChange,
    // kXEvtControllerPaired,
    // kXEvtControllerUnpaired,

    kXEvtMax,
} XEvent_t;

// extern 
// int get_xdevice_service(android::sp<IXDevInterface>& sp_ref);
/**
 * @~Chinese
 * @brief 初始化 SDK 
 * 
 * @return 0 代表成功， 负数代表错误
 */
XIM_API int xdevc_init();

/**
 * @~Chinese
 * @brief 释放SDK资源并退出
 * 
 * @return 0 代表成功， 负数代表错误
 */
XIM_API void xdevc_exit();

/**
 * @~English
 * @brief Detect whether connected to service.
 * 
 * @return returns true if connected, otherwise false.
 * 
 * @~Chinese
 * @brief 是否已连接xdeviceservice 服务端
 * 
 * @return returns true if connected, otherwise false.
 */
XIM_API bool xdevc_is_service_connected();

/**
 * @brief events callback function type 
 * @param evt event id, @see XEvent_t
 * @param args1
 * @param args2 reverse
 */
typedef void(*xdevc_events_handle_func_t)(XEvent_t evt, void* arg1, void* arg2);
/**
 * @~English
 * @brief Set events callback delegate. duplicated invoking will override callback setted
 * 
 * @param cb 
 * @return XIM_API 
 * @~Chinese
 * @brief 设置事件回调，服务端产生事件时将通过该回调通知应用。同一应用只能设置一个回调，后者覆盖前者。
 */
XIM_API void xdevc_set_events_cb(xdevc_events_handle_func_t cb);

XIM_API const char* xdevc_get_event_name(XEvent_t evt);
XIM_API const char* xdevc_get_error_name(xerror_t err);

#ifndef INVALID_HANDLE
#define INVALID_HANDLE (-1)
#endif // INVALID_HANDLE
/**
 * @brief 
 * 
 * @return  returns context instance handle, or -1 for invalid handle
 */
XIM_API xhandle_t xdevc_get_xcontext();
/**
 * @brief 
 * 
 * @return returns VPU instance handle, or -1 for invalid handle 
 */
XIM_API xhandle_t xdevc_get_vpu();
/**
 * @brief 
 * 
 * @param index 
 * @return returns controller instance handle, or -1 for invalid handle 
 */
XIM_API xhandle_t xdevc_get_controller(int index);
XIM_API int xdevc_get_number_of_controllers();

XIM_API int xdevc_do_test();

typedef enum {
    /**
     * @~Chinese
     * @brief TAG 算法
     */
    kCAlgTag = 0,       
    kCAlgIrled = 1,     ///< @~Chinese IR LED 算法
    kCAlgStream = 2,    ///< @~Chinese 数据上报功能
} CAlgType_t;

#define TAG_ALG kCAlgTag
#define LED_ALG kCAlgIrled

XIM_API const char* xdevc_get_client_version();
XIM_API int xdevc_get_client_build_number();
XIM_API const char* xdevc_get_client_build_information();
/**
 * @brief get version code of algorithm library which in client application
 * 
 * @param alg_type which algorithm. @see CAlgType_t
 * @return postive value for version, negative value for error, 
 */
XIM_API int xdevc_get_client_alg_version(int alg_type);

XIM_API const char* xdevc_get_server_version();
XIM_API int xdevc_get_server_build_number();
XIM_API const char* xdevc_get_server_build_information();
/**
 * @brief get version code of algorithm library which in service.
 * 
 * @param alg_type which algorithm. @see CAlgType_t
 * @return postive value for version, negative value for error, 
 */
XIM_API int xdevc_get_server_alg_version(int alg_type);

XIM_API int xdevc_sdk_autorize();
XIM_API int xdevc_get_authorization_available_time();


/**
 * @brief 
 *  kXTrackingModeNone
 *      Tracking is disabled, imcomplete.
 *  kXTrackingModeControllerImu
 *      Connected controller IMU data is available.
 *  kXTrackingModeTag
 *      Tag tracking is available.
 *  TRACKING_MODE_FULL
 *      All tracking function available.
 */
typedef enum {
    kXTrackingModeNone = 0,
    kXTrackingModeControllerImu = 0x01,
    kXTrackingModeTag = 0x02,

    kXTrackingModeFull = -1
} XTrackingMode_t;

/**
 * @brief Change the tracking mode of SDK.
 * 
 * @param mode @see XTrackingMode_t
 * @return  Returns 0 for success, otherwise, negative value for error.
 */
XIM_API int xdevc_set_tracking_mode(XTrackingMode_t mode);
/**
 * @brief Get current tracking mode.
 * 
 * @return Returns tracking mode value if success. 
 *  Otherwise, returns negative interge value for error.
 */
XIM_API XTrackingMode_t xdevc_get_tracking_mode();

// XIM_API int xdevc_get_paired_controllers(XControllerTypes type, int out_buffer_size, xhandle_t* out_info_buffer);
/**
 * @brief Send message data to device. Only support VPU device now.
 * 
 * @param device_handle 
 * @param size 
 * @param data 
 * @return XIM_API 
 */
XIM_API int xdevc_tx_message(xhandle_t device_handle, int size, const char* data);

/**
 * @~English
 * @brief Set Alg available
 * 
 * @param which_alg 
 * @param available 
 * @param force whether to force start  or stop
 * @return 0 for success, or negative value for error. 
 * 
 * @~Chinese
 * @brief 设置算法是否可用
 * 
 * @param which_alg 算法ID，指定操作哪一种算法
 * @param available 是否可用, true 为可用，false为不可用
 * @param start 当available为true时有效，true为自动启动算法工作
 * @return 返回0代表成功, 返回负数代表错误
 */
XIM_API int xdevc_set_alg_available(CAlgType_t which_alg, bool available, bool start);

/**
 * @brief Handle extern message, for message detail, to view protocal document.
 * 
 * @param msg 
 * @return XIM_API 
 */
XIM_API int xdevc_handle_extern_message(const char* msg);
#pragma region VPU API

/**
 * @~English
 * @defgroup vpu VPU controlling APIs
 * @~Chinese
 * @defgroup vpu VPU 控制API
 * @~
 * @{
 */

XIM_API bool xdevc_vpu_is_connected(xhandle_t vpu_handle);

// int xdevc_vpu_tx_cmd(xhandle_t vpu_hdl, const unsigned char* data, int size);
// @deprecated
XIM_API int xdevc_vpu_start_pairing_by_type(xhandle_t vpu_handle, XControllerTypes type);
// @deprecated
XIM_API int xdevc_vpu_stop_pairing(xhandle_t vpu_handle);

XIM_API int xdevc_vpu_unpair_and_disconnect_all_controllers(xhandle_t vpu_handle);

XIM_API const char* xdevc_vpu_get_firmware_version(xhandle_t vpu_handle);

XIM_API const char* xdevc_vpu_get_hardware_version(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_sn(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_device_name(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_model_name(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_fpga_version(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_ble_firmware_version(xhandle_t vpu_handle);
XIM_API const char* xdevc_vpu_get_imu_alg_version(xhandle_t vpu_handle);

XIM_API int xdevc_vpu_get_fpga_model_number(xhandle_t vpu_handle);
XIM_API int xdevc_vpu_get_mcu_model_number(xhandle_t vpu_handle);    
XIM_API int xdevc_vpu_get_ble_model_number(xhandle_t vpu_handle);


XIM_API int xdevc_vpu_start_upgrading(
    xhandle_t vpu_handle, 
    int proj_type, 
    int fw_type, 
    const char* fw_path, 
    XDeviceFirmwareUpgradeEventsDelegateFn_t event_delegate, 
    void* ud, 
    int wait_ms);
#pragma region VPU Tracking Alg
/**
 * @~English
 * @defgroup tag Tag tracking APIs
 * @~Chinese
 * @defgroup tag Tag跟踪API
 * @~
 * @{
 */

enum TagTrackingResult {
    kTagTrackingResult_NotTracking = 0,
    
	kTagTrackingResult_RotationTracked = 1,
	kTagTrackingResult_PositionTracked = 2,
	kTagTrackingResult_PoseTracked     = (kTagTrackingResult_RotationTracked | kTagTrackingResult_PositionTracked),
};
/**
 * @~English
 * @brief load marker calibration configure data from file.
 * 
 * @param vpu_handle 
 * @param file the path of file which must can be read by xdeviceservice application.
 * @param nof_output_elements size of output IDs buffer, how many id can be write in output_ids parameter.
 * @param output_ids
 * @return negative value for error, positive value for number of output tracking IDs.
 * 
 * @~Chinese
 * @brief 加载tag跟踪算法标定参数。
 * 
 * @param[in] vpu_handle            VPU 实例句柄
 * @param[in] file                  算法标定参数文件路径
 * @param[in] nof_output_elements   输出的缓存大小
 * @param[out] output_ids           输出缓存，返回标定参数中设置的跟踪ID
 * @return 返回output_ids中输出的ID个数，返回负数代表错误.
 * 
 */
XIM_API int xdevc_vpu_load_marker_calibration_from_file(xhandle_t vpu_handle, const char* file, int nof_output_elements, int* output_ids);
/**
 * @~English
 * @brief clear all loaded calibration configures of tag tracking algorithm.
 * 
 * @param vpu_handle 
 * @return 0 for success, negative value for error.
 * 
 * @~Chinese
 * @brief 清除所有已加载的Tag跟踪算法标定参数。
 * @param vpu_handle
 * @return 返回0代表成功，负数代表错误。
 */
XIM_API int xdevc_vpu_clear_marker_calibration_settings(xhandle_t vpu_handle);

/**
 * @brief 
 * 
 * @param track_id 
 * @param timestamp positive value for boot time in nanosecond to predict.
 *      0 for no predition, -1 for now timestamp,
 * @param[out] out 
 * @return returns TagTrackingResult 
 * @see TagTrackingResult 
 */
XIM_API int xdevc_vpu_get_predict_tracking(xhandle_t vpu_handle, int track_id, int64_t timestamp, XAttrTrackingInfo* out);
XIM_API int xdevc_vpu_get_tracking(xhandle_t vpu_handle, int track_id, XAttrTrackingInfo* out);
/**
 * @~English
 * @brief Get tag 6Dof information of specified tag IDs
 * 
 * @param vpu_handle 
 * @param timestamp boot time value in nanosecond for prediction.
 *      0 for no predition.
 *      -1 for predicting by current system boot timestamp.
 * @param nof_ids size of tag IDs
 * @param ids Tag ID array
 * @param nof_buf size of output buffer. 
 * @param[out] out buffer for outputing tracked tag 6dof.
 * @return number of output tracked tags, or negative value for error. 
 * 
 * @~Chinese
 * @brief 获取输入ID数组中所有ID的 tag 6Dof 信息
 * 
 * @param vpu_handle VPU 实例句柄
 * @param timestamp 预测时间，系统开机时间boot time，单位纳秒；
 *      0 代表不预测；
 *      -1 代表使用当前时间进行预测。
 * @param nof_ids 输入tag ID数组个数；
 * @param ids Tag ID 数组
 * @param nof_buf 输出缓存个数.
 * @param[out] out tag 6dof 输出缓存.
 * @return 返回跟踪到的Tag数量，或者返回负数代表错误。
 * 
 * @~
 * @code {.c}
 *      xhandle_t vpu_hdl = xdevc_get_vpu();
 *      int tag_ids[4] = { 65, 66, 67, 68, };
 *      XAttrTrackingInfo output_buffer[256];
 *      int nof_output = xdevc_vpu_get_multiple_trackings(vpu_hdl, -1, 4, tag_ids, 256, output_buffer);
 *      if (nof_output < 0) {
 *          printf("get tracked return err %d", nof_output);
 *      }
 *      assert(output_buffer[0].index == tag_ids[0]);
 *      assert(output_buffer[1].index == tag_ids[1]);
 *      assert(output_buffer[2].index == tag_ids[2]);
 *      assert(output_buffer[3].index == tag_ids[3]);
 *      
 * @endcode
 */
XIM_API int xdevc_vpu_get_multiple_trackings(xhandle_t vpu_handle, int64_t timestamp, int nof_ids, int* ids, int nof_buf, XAttrTrackingInfo* out);
/**
 * @~English
 * @brief Get tracked tag 6Dof value.
 * 
 * @param vpu_handle 
 * @param timestamp boot time value in nanosecond for prediction.
 *      0 for no predition.
 *      -1 for predicting by current system boot timestamp.
 * @param nof_buf size of output buffer.
 * @param out buffer for outputing tracked tag 6dof.
 * @return number of output tracked tags, or negative value for error.
 * 
 * @~Chinese
 * @brief 获取跟踪到的tag 6Dof信息.
 * 
 * @param vpu_handle VPU 实例句柄
 * @param timestamp 预测时间，系统开机时间boot time，单位纳秒；
 *      0 代表不预测；
 *      -1 代表使用当前时间进行预测。
 * @param nof_buf 输出缓存个数.
 * @param[out] out tag 6dof 输出缓存.
 * @return 返回跟踪到的Tag数量，或者返回负数代表错误。
 * 
 * @~
 * @code {.c}
 *      xhandle_t vpu_hdl = xdevc_get_vpu();
 *      XAttrTrackingInfo output_buffer[256];
 *      int nof_tracked = xdevc_vpu_get_tracked(vpu_hdl, -1, 256, output_buffer);
 *      if (nof_tracked < 0) {
 *          printf("get tracked return err %d", nof_tracked);
 *      }
 * @endcode
 * 
 */
XIM_API int xdevc_vpu_get_tracked(xhandle_t vpu_handle, int64_t timestamp, int nof_buf, XAttrTrackingInfo* out);

/**
 * @~English
 * @brief predict tag tracking result with providing boot timestamp in ns.
 * 
 * @param timestamp_ns  boot time value in nanosecond for prediction.
 *      0 for no predition.
 *      -1 for predicting by current system boot timestamp.
 * @return returns 0 for success, otherwise, negative value for error.
 * 
 * @~Chinese
 * @brief 预测tag跟踪, 接口执行后调用 @ref xdevc_vpu_tracking_get_predicted
 * 
 * @param timestamp_ns 预测时间，系统开机时间boot time，单位纳秒；
 *      0 代表不预测；
 *      -1 代表使用当前时间进行预测。
 * @return 返回0代表正常，返回负数代表错
 * 
 * @~
 * @see xdevc_vpu_tracking_get_predicted, xdevc_vpu_tracking_get_predicted2, xdevc_vpu_tracking_traverse_predict_result, xdevc_vpu_tracking_traverse_predict_result2
 */
XIM_API int xdevc_vpu_tracking_predict(int64_t timestamp_ns);

/**
 * @~English
 * 
 * @~Chinese
 * @brief 从预测结果获取跟踪信息, 调用前必须先调用 @ref xdevc_vpu_tracking_predict
 * 
 * @param[in] tag_id Tag ID
 * @param[out] out 6Dof输出
 * 
 * @return 返回跟踪状态 TagTrackingResult
 * 
 * @pre 调用前必须先调用 @ref xdevc_vpu_tracking_predict 并返回0
 * 
 * @~
 * @see xdevc_vpu_tracking_predict
 * 
 * @code {.c}
 *      int pred_err = xdevc_vpu_tracking_predict(0);
 *      if (pred_err == 0) {
 *          int tag_id = 65;
 *          XAttrTrackingInfo info = {0};
 *          int track_st = xdevc_vpu_tracking_get_predicted(tag_id, &info);
 *      }
 * @endcode
 * 
 */
XIM_API int xdevc_vpu_tracking_get_predicted(int tag_id, XAttrTrackingInfo* out);

/**
 * @~Chinese
 * 
 * @brief 从预测结果获取跟踪信息， 调用前必须先调用 @ref xdevc_vpu_tracking_predict
 * 
 * @param[in] tag_id Tag ID
 * @param[out] px 
 * @param[out] py 
 * @param[out] pz 
 * @param[out] qx 
 * @param[out] qy 
 * @param[out] qz 
 * @param[out] qw 
 * @param[out] timestamp 
 * @param[out] confidence 
 * @param[out] marker_distance 
 * @return 返回跟踪状态 TagTrackingResult
 * 
 * @pre 调用前必须先调用 @ref xdevc_vpu_tracking_predict 并返回0
 * @~
 * @see xdevc_vpu_tracking_predict, xdevc_vpu_tracking_get_predicted
 * 
 * @code {.c}
 *      int pred_err = xdevc_vpu_tracking_predict(0);
 *      if (pred_err == 0) {
 *          int tag_id = 65;
 * 
 *          float px, py, pz;
 *          float qx, qy, qz, qw;
 *          uint64_t timestamp;
 *          double confidence, marker_distance;
 * 
 *          int track_st = xdevc_vpu_tracking_get_predicted2(tag_id,
 *              &px, &py, &pz,
 *              &qx, &qy, &qz, &qw,
 *              &timestamp,
 *              &confidence,
 *              &marker_distance);
 *      }
 * @endcode
 */
XIM_API int xdevc_vpu_tracking_get_predicted2(
    int tag_id, 
    float* px, float* py, float* pz,
    float* qx, float* qy, float* qz, float* qw,
    uint64_t* timestamp,
    double* confidence,
    double* marker_distance);

/**
 * @~Chinese
 * @brief 遍历获取所有有跟踪的预测结果。
 *  调用前须调用 @ref xdevc_vpu_tracking_predict
 *  每调用一次该接口后游标加1，直到全部遍历完成，
 *  调用 @ref xdevc_vpu_tracking_predict更新预测结果并重置游标。
 * 
 * @param[out] out 输出 @ref XAttrTrackingInfo 类型
 * @return 返回跟踪状态，当一次预测的所有结果遍历完成后返回-1。
 * 
 * @pre 调用前必须先调用 @ref xdevc_vpu_tracking_predict 并返回0
 * 
 * @~
 * @see xdevc_vpu_tracking_predict, xdevc_vpu_tracking_traverse_predict_result
 * 
 * @code {.c}
 *      int pred_err = xdevc_vpu_tracking_predict(0);
 *      if (pred_err >= 0) {
 *          XAttrTrackingInfo info = {0};
 * 
 *          int track_st = 0;
 *          do {
 *              track_st = xdevc_vpu_tracking_traverse_predict_result(&info);
 *          } while(track_st >= 0);
 *      }
 * @endcode
 */
XIM_API int xdevc_vpu_tracking_traverse_predict_result(XAttrTrackingInfo* out);

/**
 * @~Chinese
 * @brief 遍历获取所有有跟踪的预测结果。
 *  调用前须调用 @ref xdevc_vpu_tracking_predict
 *  每调用一次该接口后游标加1，直到全部遍历完成，
 *  调用 @ref xdevc_vpu_tracking_predict更新预测结果并重置游标。
 * 
 * @param[out] tag_id 
 * @param[out] px       位置x
 * @param[out] py       位置y
 * @param[out] pz       位置z
 * @param[out] qx       姿态四元素x
 * @param[out] qy       姿态四元素y
 * @param[out] qz       姿态四元素z
 * @param[out] qw       姿态四元素w
 * @param[out] timestamp 
 * @param[out] confidence 置信度，0到1
 * @param[out] marker_distance 
 * @return 返回跟踪状态，当一次预测的所有结果遍历完成后返回-1。
 * 
 * @pre 调用前必须先调用 @ref xdevc_vpu_tracking_predict 并返回0
 * 
 * @~
 * @see xdevc_vpu_tracking_predict, xdevc_vpu_tracking_traverse_predict_result
 * 
 * @code {.c}
 *      int pred_err = xdevc_vpu_tracking_predict(0);
 *      if (pred_err >= 0) {
 *          int tag_id;
 * 
 *          float px, py, pz;
 *          float qx, qy, qz, qw;
 *          uint64_t timestamp;
 *          double confidence, marker_distance;
 * 
 *          int track_st = 0;
 *          do {
 *              track_st = xdevc_vpu_tracking_traverse_predict_result2(
 *                  &tag_id,
 *                  &px, &py, &pz,
 *                  &qx, &qy, &qz, &qw,
 *                  &timestamp,
 *                  &confidence,
 *                  &marker_distance);
 *          
 *          } while(track_st >= 0);
 *      }
 * @endcode
 */
XIM_API int xdevc_vpu_tracking_traverse_predict_result2(
    int* tag_id,
    float* px, float* py, float* pz,
    float* qx, float* qy, float* qz, float* qw,
    uint64_t* timestamp,
    double* confidence,
    double* marker_distance);

/**
 * @}
 * end of group Tag
 */
#pragma endregion VPU Tracking Alg


#define VPU_POWER_MODE_WAKEUP 0x02
#define VPU_POWER_MODE_SLEEP 0x01
#define VPU_POWER_MODE_SLEEP2 0x04

/**
 * @brief change VPU power mode
 * 
 * @param handle 
 * @param power_mode 
 *      @see VPU_POWER_MODE_WAKEUP, VPU_POWER_MODE_SLEEP, VPU_POWER_MODE_SLEEP2
 * @return int 
 */
XIM_API int xdevc_vpu_set_power_mode(xhandle_t handle, int power_mode);

/**
 * @brief VPU up camera frequence configure
 * 
 */
enum XVpuCameraFreq {
    kVpuCamFreq_default = -1, ///< default value is not to set
    kVpuCamFreq_60 = 0x00, ///< 60fps
    kVpuCamFreq_30 = 0x01, ///< 30fps
    kVpuCamFreq_20 = 0x02, ///< 20fps
    kVpuCamFreq_15 = 0x03, ///< 15fps
    kVpuCamFreq_12 = 0x04, ///< 12fps
    kVpuCamFreq_10 = 0x05, ///< 10fps
    kVpuCamFreq_6 = 0x09, ///< 6fps
    kVpuCamFreq_5 = 0x0A, ///< 5fps
    kVpuCamFreq_1 = 0x3B, ///< 1fps
};
/**
 * @brief 
 * 
 * @param handle 
 * @param available 
 * @param freq camera frequence, supported values: 1,5,6,10,12,15,20,30,60. Set 0 for default.
 * @return XIM_API 
 */
XIM_API int xdevc_vpu_set_up_camera_config(xhandle_t handle, bool available, int freq);

/**
 * @ingroup client_1_3_2_33
 */
enum XGlassConfigType {
    /**
     * @~Chinese
     * @brief 设置百分比
     */
    kXGlassPerspectivePercentage = 0,
    /**
     * @~Chinese
     * @brief 设置电压，数值单位mV
     */
    kXGlassVoltage = 1,
};

/**
 * @ingroup client_1_3_2_33
 * 
 * @~English
 * @brief change glass perspective
 * 
 * @param handle 
 * @param what 
 * @param val 
 * @return XIM_API 
 * 
 * @~Chinese
 * 
 * @brief 设置电致变色
 * 
 * @param handle 
 * @param what  设置参数的类型， @ref XGlassConfigType
 * @param val   设置参数值，具体数值范围请查阅协议文档。
 * @return  
 * 
 * @test 测试命令 ./run.sh vpu glass -h 
 * @test 设置百分比修改透明度, 20%: ./run.sh vpu glass percentage 20
 * @test 设置电压修改透明度, 电压200mV: ./run.sh vpu glass voltage 200
 * 
 * @~
 * 
 * @code {.cpp}
 *  auto vpu_hdl = xdevc_get_vpu();
 *  if (vpu_hdl > 0) {
 *      int rs = xdevc_vpu_set_glass_perspective(vpu_hdl, kXGlassPerspectivePercentage, 50);
 *      std::cerr << "rs " << xdevc_get_error_name(rs) << "(" << rs << ")" << std::endl;
 *  }
 * @endcode
 * 
 */
XIM_API int xdevc_vpu_set_glass_perspective(xhandle_t handle, enum XGlassConfigType what, int val);

typedef enum XAmbientLedType {
    kXAmbientLedType_Left = 0x00,
    kXAmbientLedType_Right = 0x01,
    // kXAmbientLedType_LeftAndRight = 0x02,
} XAmbientLedType_t;
// typedef enum XStatusLedAction {
//     kXStatusLedAction_CloseAll = 0x00,
//     kXStatusLedAction_LightRed = 0x01,
//     kXStatusLedAction_LightGreen = 0x02,
//     kXStatusLedAction_LightRedAndGreen = 0x03
// } XStatusLedAction_t;
typedef enum XAmbientLedAction {
    kXAmbientLedAction_Close = 0x00,
    kXAmbientLedAction_Light = 0x01,
    kXAmbientLedAction_Breathing = 0x02,
    kXAmbientLedAction_Flash = 0x03
} XAmbientLedAction_t;
typedef enum XAmbientLedColor {
    kXAmbientLedColor_Red = 0x00,
    kXAmbientLedColor_Green = 0x01,
    kXAmbientLedColor_Blue = 0x03,
    kXAmbientLedColor_Orange = 0x04,
    kXAmbientLedColor_Yellow = 0x05,
    kXAmbientLedColor_Cyan = 0x06,
    kXAmbientLedColor_Purple = 0x07,
    kXAmbientLedColor_White = 0x08,
    kXAmbientLedColor_Custom = 0x09,
} XAmbientLedColor_t;

typedef struct XAmbientLedSettings {
    uint8_t color;
    uint8_t red_strength;
    uint8_t green_strength;
    uint8_t blue_strength;
    uint16_t period_us;
} XAmbientLedSettings_t;

typedef struct XAmbientLedArg {
    XAmbientLedType_t type;
    XAmbientLedAction_t action;
    XAmbientLedSettings_t settings;
} XAmbientLedArg_t;

/**
 * @brief change ambient LED light status.
 * 
 * @param handle 
 * @param nof_args 
 * @param args 
 * @return XIM_API 
 */
XIM_API int xdevc_vpu_control_led(xhandle_t handle, int nof_args, XAmbientLedArg_t* args);

#pragma region VPU Factory Test

#define VPU_CAMERA_DOWN_TAG     0
#define VPU_CAMERA_LEFT_VIO     2
#define VPU_CAMERA_RIGHT_VIO    1 
#define VPU_CAMERA_UP_TAG       3

#define VPU_IMAGE_TYPE_RAW      0
#define VPU_IMAGE_TYPE_BINARY   1

/**
 * @brief Change camera output image type.
 * 
 * @param which_cam which camera @see VPU_CAMERA_DOWN_TAG VPU_CAMERA_LEFT_VIO VPU_CAMERA_RIGHT_VIO VPU_CAMERA_UP_TAG
 * @param type image type , 0 for raw image, and 1 for binary image.
 * @return XIM_API 
 */
XIM_API int xdevc_vpu_set_camera_image_type(xhandle_t vpu_handle, int which_cam, int type);

/**
 * @brief 
 * 
 * @param available set true to enable VIO in VPU.
 * @param exposure_level VIO camera exposure config, 0 for low level, 1 for high level.
 * @return XIM_API 
 */
XIM_API int xdevc_vpu_set_aio_alg_exposure(xhandle_t vpu_handle, bool available, int exposure_level);
XIM_API int xdevc_vpu_write_fpga_registry(xhandle_t vpu_handle, int address, int value);
XIM_API int xdevc_vpu_read_fpga_registry(xhandle_t vpu_handle, int address, int* value);
XIM_API int xdevc_vpu_write_camera_registry(xhandle_t vpu_handle, uint8_t camera_index, uint16_t address, uint16_t value);
XIM_API int xdevc_vpu_read_camera_registry(xhandle_t handle, uint8_t camera_index, uint16_t address, uint16_t* value);
XIM_API int xdevc_vpu_clear_camera_calibration(xhandle_t handle);
XIM_API int xdevc_vpu_write_camera_calibration(xhandle_t handle, int rotate_angle, const char* calib_data_file);
XIM_API int xdevc_vpu_read_camera_calibration(xhandle_t handle, const char* output_calib_file);
XIM_API int xdevc_vpu_clear_ir_led_calibration(xhandle_t handle);
XIM_API int xdevc_vpu_write_ir_led_calibration(xhandle_t handle, int rotate_angle, const char* src_data_file);
XIM_API int xdevc_vpu_read_ir_led_calibration(xhandle_t handle, const char* output_calib_file);
XIM_API int xdevc_vpu_set_ble_soc_rf(xhandle_t handle, bool enable, int freqency, int power);
XIM_API int xdevc_vpu_set_aging_test_mode(xhandle_t handle, bool enable, int aging_mode_usb_pid);

#pragma endregion VPU Factory

/**
 * @}
 * end of vpu group
 */
#pragma endregion VPU API

#pragma region Controller API
/**
 * @defgroup ctrl Controller API
 * @{
 */

XIM_API XConnectionStates xdevc_ctrl_get_connect_state(xhandle_t controller_handle);
XIM_API bool xdevc_ctrl_is_paired(xhandle_t controller_handle);
XIM_API XPairingStates  xdevc_ctrl_get_pairing_state(xhandle_t controller_handle);

XIM_API int xdevc_ctrl_get_imu_fps(xhandle_t controller_handle);
XIM_API int xdevc_ctrl_get_battery_level(xhandle_t controller_handle);
XIM_API int xdevc_ctrl_get_button_state_bitmask(xhandle_t controller_handle);
XIM_API int xdevc_ctrl_get_trigger(xhandle_t controller_handle);
XIM_API int xdevc_ctrl_get_imu(xhandle_t controller_handle, XAttrImuInfo* imu);
XIM_API int xdevc_ctrl_get_fusion(xhandle_t controller_handle, int64_t timestamp_ns, XAttrTrackingInfo* track_info);
XIM_API int xdevc_ctrl_get_touchpad_state(xhandle_t controller_handle, XAttrTouchPadState* state);
XIM_API int xdevc_ctrl_get_controller_state(xhandle_t controller_handle, XAttrControllerState* state);

XIM_API int xdevc_ctrl_get_track_id(xhandle_t controller_handle);
XIM_API int xdevc_ctrl_get_device_type(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_device_address(xhandle_t controller_handle);

XIM_API const char* xdevc_ctrl_get_device_name(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_software_revision(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_hardware_revision(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_serial_number(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_model_name(xhandle_t controller_handle);
XIM_API const char* xdevc_ctrl_get_manufacture_name(xhandle_t controller_handle);


XIM_API xerror_t xdevc_ctrl_connect(xhandle_t controller_handle);
XIM_API xerror_t xdevc_ctrl_connect_to_type(xhandle_t controller_handle, int type, bool force);
/**
 * @brief 
 * 
 * @param controller_handle 
 * @param mac MAC string of controller, e.g. 01:09:0B:09:0E:39
 * @param force 
 * @return XIM_API 
 */
XIM_API xerror_t xdevc_ctrl_connect_to_address(xhandle_t controller_handle, const char* mac, bool force);
XIM_API xerror_t xdevc_ctrl_connect_to_rfid(xhandle_t controller_handle, int rfid, bool force);
XIM_API xerror_t xdevc_ctrl_connect_to_rfid_pattern(xhandle_t controller_handle, int rfid_pattern, bool force);
XIM_API xerror_t xdevc_ctrl_disconnect(xhandle_t controller_handle);
XIM_API xerror_t xdevc_ctrl_confirm_pair(xhandle_t controller_handle);
XIM_API xerror_t xdevc_ctrl_hold_connection(xhandle_t controller_handle, int hold_time_sec);
// XIM_API xerror_t xdevc_ctrl_bind(xhandle_t controller_handle, int type, const char* mac);
XIM_API xerror_t xdevc_ctrl_unbind(xhandle_t controller_handle);
XIM_API xerror_t xdevc_ctrl_vibrate(xhandle_t controller_handle, int strengh_percentage, int dur_ms);

/**
 * @brief Read controller state raw data received from accessory.
 * 
 * @param controller_handle which controller
 * @param buf_size sizeof buffer
 * @param[out] buf output data buffer
 * @param offset read offset.
 * @param read_size to read size.
 * @return return negatvie value for error, otherwise return read size
 */
XIM_API xerror_t xdevc_ctrl_get_state_raw_data(xhandle_t controller_handle, size_t buf_size, uint8_t* buf, int offset, int read_size);

/**
 * @brief 
 * 
 * @param controller_handle 
 * @param uuid 
 * @param buf_size 
 * @param buf 
 * @return XIM_API 
 */
XIM_API xerror_t xdevc_ctrl_tx_message(xhandle_t controller_handle, int uuid, size_t msg_size, uint8_t* msg);

#pragma region gun controller
/**
 * @brief only available while connecting to gun type accessory.
 * 
 * @param controller_handle 
 * @return XIM_API 
 */
XIM_API int xdevc_ctrl_get_gun_left_ammo_number(xhandle_t controller_handle);
/**
 * @brief only available while connecting to gun type accessory.
 * 
 * @param controller_handle 
 * @return XIM_API 
 */
XIM_API int xdevc_ctrl_get_gun_remaining_life(xhandle_t controller_handle);

#pragma endregion gun controller

/**
 * @}
 * end of group ctrl
 */
#pragma endregion Controller API


#pragma region Others
XIM_API xerror_t xdevc_start_vio_service();
XIM_API xerror_t xdevc_stop_vio_service();
#pragma endregion Others

#ifdef __cplusplus
}
#endif // __cplusplus


#endif // __XDEVICE_CLIENT_H__
